from .api import Cluster, Document, Index
from .session import ElasticsearchSession


class Elasticsearch:
    _cluster = None
    _index = None
    _document = None

    def __init__(self, host):
        self.session = ElasticsearchSession(host)

        self.cluster = Cluster(self.session)
        self.index = Index(self.session)
        self.document = Document(self.session)


def connect(host):
    return Elasticsearch(host)



import requests


class ElasticsearchError(Exception):
    pass


class Cluster:

    def __init__(self, session):
        self.session = session


class Index:

    def __init__(self, session):
        self.session = session

    def exists(self, name):
        response = self.session.head(name)
        if response.status_code == requests.codes.ok:
            return True
        if response.status_code == requests.codes.not_found:
            return False
        raise ElasticsearchError(f'{response.status_code}: {response.url}')

    def create(self, name):
        response = self.session.put(name)
        if response.status_code == requests.codes.ok:
            return response.json()
        raise ElasticsearchError(response.text)

    def delete(self, name):
        response = self.session.delete(name)
        if response.status_code == requests.codes.ok:
            return response.json()
        raise ElasticsearchError(response.text)

    def get(self, name, id):
        response = self.session.get(name, '_doc', id)
        if response.status_code == requests.codes.ok:
            return response.json()['_source']
        raise ElasticsearchError(response.text)

    def serialize(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            result = func(*args, **kwargs)
            return {
                'total': result['hits']['total']['value'],
                'data': [x['_source'] for x in result['hits']['hits']]
            }
        return wrapper

    @serialize
    def search(self, name):
        response = self.session.get(name, '_search')
        if response.status_code == requests.codes.ok:
            return response.json()
        raise ElasticsearchError(response.text)


class Document:

    def __init__(self, session):
        self.session = session

    def create(self, name, doc, id=None, refresh=False):
        params = None
        if refresh:
            params = {'refresh': 'true'}
        if id:
            response = self.session.put(name, '_doc', id, json=doc, params=params)
        else:
            # Automatic ID Generation
            response = self.session.post(name, '_doc/', json=doc, params=params)
        if response.status_code == requests.codes.created:
            return response.json()
        raise ElasticsearchError(f'{response.status_code}: {response.text}')

    def bulk_create(self, name, bulk_data):
        buf = []
        for data in bulk_data:
            buf.append({'index': {}})
            buf.append(data)
        ndjson = '\n'.join([json.dumps(x, separators=(',', ':')) for x in buf])
        ndjson += '\n'
        headers = {'Content-Type': 'application/x-ndjson'}

        response = self.session.post(name, '_bulk', data=ndjson, headers=headers)
        if response.status_code == requests.codes.ok:
            return response.json()
        raise ElasticsearchError(response.text)



import requests
from functools import wraps
import urllib3

urllib3.disable_warnings()


class ElasticsearchSession(requests.Session):

    def __init__(self, host):
        '''
        Argument
          host: {protocol}://{domain}:{port}
        '''
        super().__init__()
        self.host = host
        self.verify = False

    def __del__(self):
        self.close()

    def urljoin(func):
        @wraps(func)
        def wrapper(self, *paths, **kwargs):
            paths = '/'.join((str(path) for path in paths))
            url = f'{self.host}/{paths}'
            return func(self, url, **kwargs)
        return wrapper

    @urljoin
    def head(self, url, **kwargs):
        return super().head(url=url, **kwargs)

    @urljoin
    def get(self, url, **kwargs):
        return super().get(url=url, **kwargs)

    @urljoin
    def post(self, url, **kwargs):
        return super().post(url=url, **kwargs)

    @urljoin
    def put(self, url, **kwargs):
        return super().put(url=url, **kwargs)

    @urljoin
    def delete(self, url, **kwargs):
        return super().delete(url=url, **kwargs)
